/**
 * Copyright (c) 2008-2023, MOVES Institute, Naval Postgraduate School (NPS). All rights reserved.
 * This work is provided under a BSD-style open-source license, see project
 * <a href="https://savage.nps.edu/opendis7-java/license.html" target="_blank">license.html</a> and <a href="https://savage.nps.edu/opendis7-java/license.txt" target="_blank">license.txt</a>
 */
// header autogenerated using string template dis7javalicense.txt



package edu.nps.moves.dis7.pdus;

import java.util.*;
import java.io.*;
import edu.nps.moves.dis7.enumerations.*;
import java.nio.ByteBuffer;

/**
 * 7.10.6 Used to communicate detailed information about the addition/modification of a synthetic environment object that is geometrically anchored to the terrain with a set of three or more points that come to a closure.
 * @see <a href="https://ieeexplore.ieee.org/document/6387564" target="_blank">IEEE Std 1278.1-2012, IEEE Standard for Distributed Interactive Simulation - Application Protocols</a> 
 */
public class ArealObjectStatePdu extends SyntheticEnvironmentFamilyPdu implements Serializable
{
   /** Object in synthetic environment */
   protected ObjectIdentifier  objectID = new ObjectIdentifier(); 

   /** Object with which this point object is associated */
   protected ObjectIdentifier  referencedObjectID = new ObjectIdentifier(); 

   /** unique update number of each state transition of an object */
   protected short updateNumber;

   /** force ID provides a unique identifier uid 6 */
   protected ForceID forceID = ForceID.values()[0];

   /** modifications enumeration uid 242 */
   protected ObjectStateModificationArealObject modifications = new ObjectStateModificationArealObject();

   /** Object type */
   protected ObjectType  objectType = new ObjectType(); 

   /** Object appearance */
   protected int specificObjectAppearance;

   /** Object appearance */
   protected short generalObjectAppearance;

   /** Number of points */
   protected short numberOfPoints;

   /** requesterID */
   protected SimulationAddress  requesterID = new SimulationAddress(); 

   /** receiver ID provides a unique identifier */
   protected SimulationAddress  receivingID = new SimulationAddress(); 

   /** location of object */
   protected List< Vector3Double > objectLocation = new ArrayList<>();
 

/** Constructor creates and configures a new instance object */
 public ArealObjectStatePdu()
 {
    setPduType( DisPduType.AREAL_OBJECT_STATE );
 }
/** copy method creates a deep copy of current object using preferred marshalling method
 * @return deep copy of PDU */
 public synchronized ArealObjectStatePdu copy()
 {
     return copyDataOutputStream();
 }
/** Creates a "deep copy" of current object using ByteBuffer methods.
 * @return deep copy of PDU */
 public synchronized ArealObjectStatePdu copyByteBuffer()
 {
     ArealObjectStatePdu newCopy = new ArealObjectStatePdu();
     ByteBuffer byteBuffer = ByteBuffer.allocate(400);
     try
     {
         this.marshal(byteBuffer);      // working
         newCopy.unmarshal(byteBuffer); // not working
     }
     catch (Exception e)
     {
         System.err.println("ArealObjectStatePdu deep copy() marshall/unmarshall ByteBuffer exception " + e.getMessage());
         e.printStackTrace(System.err);
         System.exit(-1); // TODO: Abruptly ending VM not a good practice
     }
     return newCopy;
 }

/** byteArrayOutputStream (baos) is used for marshal/unmarshal serialization
   * @see copyDataOutputStream() */
protected ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
/** dataOutputStream (dos) is used for marshal/unmarshal serialization
   * @see copyDataOutputStream() */
protected DataOutputStream      dataOutputStream      = new DataOutputStream(byteArrayOutputStream);

/** copy method creates a deep copy of current object using DataOutputStream methods.
 * @return deep copy of PDU */
 public synchronized ArealObjectStatePdu copyDataOutputStream()
 {
     ArealObjectStatePdu newCopy = new ArealObjectStatePdu();
     try
     {
         this.marshal(dataOutputStream);
         byte[] byteArrayDOS = byteArrayOutputStream.toByteArray();
         newCopy.unmarshal(ByteBuffer.wrap(byteArrayDOS));
     }
     catch (Exception e)
     {
         System.err.println("ArealObjectStatePdu deep copy() marshall/unmarshall DataOutputStream exception " + e.getMessage());
         e.printStackTrace(System.err);
         System.exit(-1); // TODO: Abruptly ending VM not a good practice
     }
        try
        {
                 dataOutputStream.flush();
            byteArrayOutputStream.flush();
            byteArrayOutputStream.reset();
        }
        catch (IOException ioe)
        {
            System.err.println("ArealObjectStatePdu copyDataOutputStream() flush IOException: " + ioe.getMessage());
        }
     return newCopy;
 }

  /**
   * Returns size of this serialized (marshalled) object in bytes
   * @see <a href="https://en.wikipedia.org/wiki/Marshalling_(computer_science)" target="_blank">https://en.wikipedia.org/wiki/Marshalling_(computer_science)</a>
   * @return serialized size in bytes
   */
public int getMarshalledSize()
{
   int marshalSize = 0; 

   marshalSize = super.getMarshalledSize();
   if (objectID != null)
       marshalSize += objectID.getMarshalledSize();
   if (referencedObjectID != null)
       marshalSize += referencedObjectID.getMarshalledSize();
   marshalSize += 2;  // updateNumber
   if (forceID != null)
       marshalSize += forceID.getMarshalledSize();
   if (modifications != null)
       marshalSize += modifications.getMarshalledSize();
   if (objectType != null)
       marshalSize += objectType.getMarshalledSize();
   marshalSize += 4;  // specificObjectAppearance
   marshalSize += 2;  // generalObjectAppearance
   marshalSize += 2;  // numberOfPoints
   if (requesterID != null)
       marshalSize += requesterID.getMarshalledSize();
   if (receivingID != null)
       marshalSize += receivingID.getMarshalledSize();
   if (objectLocation != null)
       for (int idx=0; idx < objectLocation.size(); idx++)
       {
            Vector3Double listElement = objectLocation.get(idx);
            marshalSize += listElement.getMarshalledSize();
       }

   return marshalSize;
}


/** Setter for {@link ArealObjectStatePdu#objectID}
  * @param pObjectID new value of interest
  * @return same object to permit progressive setters */
public synchronized ArealObjectStatePdu setObjectID(ObjectIdentifier pObjectID)
{
    objectID = pObjectID;
    return this;
}
/** Getter for {@link ArealObjectStatePdu#objectID}
  * @return value of interest */
public ObjectIdentifier getObjectID()
{
    return objectID;
}


/** Setter for {@link ArealObjectStatePdu#referencedObjectID}
  * @param pReferencedObjectID new value of interest
  * @return same object to permit progressive setters */
public synchronized ArealObjectStatePdu setReferencedObjectID(ObjectIdentifier pReferencedObjectID)
{
    referencedObjectID = pReferencedObjectID;
    return this;
}
/** Getter for {@link ArealObjectStatePdu#referencedObjectID}
  * @return value of interest */
public ObjectIdentifier getReferencedObjectID()
{
    return referencedObjectID;
}


/** Setter for {@link ArealObjectStatePdu#updateNumber}
  * @param pUpdateNumber new value of interest
  * @return same object to permit progressive setters */
public synchronized ArealObjectStatePdu setUpdateNumber(short pUpdateNumber)
{
    updateNumber = pUpdateNumber;
    return this;
}
/** Utility setter for {@link ArealObjectStatePdu#updateNumber}
  * @param pUpdateNumber new value of interest
  * @return same object to permit progressive setters */
public synchronized ArealObjectStatePdu setUpdateNumber(int pUpdateNumber){
    updateNumber = (short) pUpdateNumber;
    return this;
}
/** Getter for {@link ArealObjectStatePdu#updateNumber}
  * @return value of interest */
public short getUpdateNumber()
{
    return updateNumber; 
}

/** Setter for {@link ArealObjectStatePdu#forceID}
  * @param pForceID new value of interest
  * @return same object to permit progressive setters */
public synchronized ArealObjectStatePdu setForceID(ForceID pForceID)
{
    forceID = pForceID;
    return this;
}
/** Getter for {@link ArealObjectStatePdu#forceID}
  * @return value of interest */
public ForceID getForceID()
{
    return forceID; 
}

/** Setter for {@link ArealObjectStatePdu#modifications}
  * @param pModifications new value of interest
  * @return same object to permit progressive setters */
public synchronized ArealObjectStatePdu setModifications(ObjectStateModificationArealObject pModifications)
{
    modifications = pModifications;
    return this;
}
/** Getter for {@link ArealObjectStatePdu#modifications}
  * @return value of interest */
public ObjectStateModificationArealObject getModifications()
{
    return modifications; 
}

/** Setter for {@link ArealObjectStatePdu#objectType}
  * @param pObjectType new value of interest
  * @return same object to permit progressive setters */
public synchronized ArealObjectStatePdu setObjectType(ObjectType pObjectType)
{
    objectType = pObjectType;
    return this;
}
/** Getter for {@link ArealObjectStatePdu#objectType}
  * @return value of interest */
public ObjectType getObjectType()
{
    return objectType;
}


/** Setter for {@link ArealObjectStatePdu#specificObjectAppearance}
  * @param pSpecificObjectAppearance new value of interest
  * @return same object to permit progressive setters */
public synchronized ArealObjectStatePdu setSpecificObjectAppearance(int pSpecificObjectAppearance)
{
    specificObjectAppearance = pSpecificObjectAppearance;
    return this;
}
/** Getter for {@link ArealObjectStatePdu#specificObjectAppearance}
  * @return value of interest */
public int getSpecificObjectAppearance()
{
    return specificObjectAppearance; 
}

/** Setter for {@link ArealObjectStatePdu#generalObjectAppearance}
  * @param pGeneralObjectAppearance new value of interest
  * @return same object to permit progressive setters */
public synchronized ArealObjectStatePdu setGeneralObjectAppearance(short pGeneralObjectAppearance)
{
    generalObjectAppearance = pGeneralObjectAppearance;
    return this;
}
/** Utility setter for {@link ArealObjectStatePdu#generalObjectAppearance}
  * @param pGeneralObjectAppearance new value of interest
  * @return same object to permit progressive setters */
public synchronized ArealObjectStatePdu setGeneralObjectAppearance(int pGeneralObjectAppearance){
    generalObjectAppearance = (short) pGeneralObjectAppearance;
    return this;
}
/** Getter for {@link ArealObjectStatePdu#generalObjectAppearance}
  * @return value of interest */
public short getGeneralObjectAppearance()
{
    return generalObjectAppearance; 
}

/** Setter for {@link ArealObjectStatePdu#requesterID}
  * @param pRequesterID new value of interest
  * @return same object to permit progressive setters */
public synchronized ArealObjectStatePdu setRequesterID(SimulationAddress pRequesterID)
{
    requesterID = pRequesterID;
    return this;
}
/** Getter for {@link ArealObjectStatePdu#requesterID}
  * @return value of interest */
public SimulationAddress getRequesterID()
{
    return requesterID;
}


/** Setter for {@link ArealObjectStatePdu#receivingID}
  * @param pReceivingID new value of interest
  * @return same object to permit progressive setters */
public synchronized ArealObjectStatePdu setReceivingID(SimulationAddress pReceivingID)
{
    receivingID = pReceivingID;
    return this;
}
/** Getter for {@link ArealObjectStatePdu#receivingID}
  * @return value of interest */
public SimulationAddress getReceivingID()
{
    return receivingID;
}


/** Setter for {@link ArealObjectStatePdu#objectLocation}
  * @param pObjectLocation new value of interest
  * @return same object to permit progressive setters */
public synchronized ArealObjectStatePdu setObjectLocation(List<Vector3Double> pObjectLocation)
{
    objectLocation = pObjectLocation;
    return this;
}
/** Getter for {@link ArealObjectStatePdu#objectLocation}
  * @return value of interest */
public List<Vector3Double> getObjectLocation()
{
    return objectLocation; 
}

/**
 * Serializes an object to a DataOutputStream.
 * @throws java.lang.Exception if something goes wrong
 * @see java.io.DataOutputStream
 * @param dos the OutputStream
 */
public void marshal(DataOutputStream dos) throws Exception
{
    super.marshal(dos);
    try 
    {
       objectID.marshal(dos);
       referencedObjectID.marshal(dos);
       dos.writeShort(updateNumber);
       forceID.marshal(dos);
       modifications.marshal(dos);
       objectType.marshal(dos);
       dos.writeInt(specificObjectAppearance);
       dos.writeShort(generalObjectAppearance);
       dos.writeShort(objectLocation.size());
       requesterID.marshal(dos);
       receivingID.marshal(dos);

       for (int idx = 0; idx < objectLocation.size(); idx++)
       {
            Vector3Double aVector3Double = objectLocation.get(idx);
            aVector3Double.marshal(dos);
       }

    }
    catch(Exception e)
    {
      System.err.println(e);
    }
}

/**
 * Deserializes an object from a DataInputStream.
 * @throws java.lang.Exception if something goes wrong
 * @see java.io.DataInputStream
 * @see <a href="https://en.wikipedia.org/wiki/Marshalling_(computer_science)" target="_blank">https://en.wikipedia.org/wiki/Marshalling_(computer_science)</a>
 * @param dis the InputStream
 * @return marshalled serialized size in bytes
 */
public synchronized int unmarshal(DataInputStream dis) throws Exception
{
    int uPosition = 0;
    uPosition += super.unmarshal(dis);

    try 
    {
        uPosition += objectID.unmarshal(dis);
        uPosition += referencedObjectID.unmarshal(dis);
        updateNumber = (short)dis.readUnsignedShort();
        uPosition += 2;
        forceID = ForceID.unmarshalEnum(dis);
        uPosition += forceID.getMarshalledSize();
        uPosition += modifications.unmarshal(dis);
        uPosition += objectType.unmarshal(dis);
        specificObjectAppearance = dis.readInt();
        uPosition += 4;
        generalObjectAppearance = (short)dis.readUnsignedShort();
        uPosition += 2;
        numberOfPoints = (short)dis.readUnsignedShort();
        uPosition += 2;
        uPosition += requesterID.unmarshal(dis);
        uPosition += receivingID.unmarshal(dis);
        for (int idx = 0; idx < numberOfPoints; idx++)
        {
            Vector3Double anX = new Vector3Double();
            uPosition += anX.unmarshal(dis);
            objectLocation.add(anX);
        }

    }
    catch(Exception e)
    { 
      System.err.println(e); 
    }
    return getMarshalledSize();
}

/**
 * Packs an object into the ByteBuffer.
 * @throws java.nio.BufferOverflowException if byteBuffer is too small
 * @throws java.nio.ReadOnlyBufferException if byteBuffer is read only
 * @see java.nio.ByteBuffer
 * @param byteBuffer The ByteBuffer at the position to begin writing
 * @throws Exception ByteBuffer-generated exception
 */
public void marshal(java.nio.ByteBuffer byteBuffer) throws Exception
{
   super.marshal(byteBuffer);
   objectID.marshal(byteBuffer);
   referencedObjectID.marshal(byteBuffer);
   byteBuffer.putShort( (short)updateNumber);
   forceID.marshal(byteBuffer);
   modifications.marshal(byteBuffer);
   objectType.marshal(byteBuffer);
   byteBuffer.putInt( (int)specificObjectAppearance);
   byteBuffer.putShort( (short)generalObjectAppearance);
   byteBuffer.putShort( (short)objectLocation.size());
   requesterID.marshal(byteBuffer);
   receivingID.marshal(byteBuffer);

   for (int idx = 0; idx < objectLocation.size(); idx++)
   {
        Vector3Double aVector3Double = objectLocation.get(idx);
        aVector3Double.marshal(byteBuffer);
   }

}

/**
 * Unpacks a Pdu from the underlying data.
 * @throws java.nio.BufferUnderflowException if byteBuffer is too small
 * @see java.nio.ByteBuffer
 * @see <a href="https://en.wikipedia.org/wiki/Marshalling_(computer_science)" target="_blank">https://en.wikipedia.org/wiki/Marshalling_(computer_science)</a>
 * @param byteBuffer The ByteBuffer at the position to begin reading
 * @return marshalled serialized size in bytes
 * @throws Exception ByteBuffer-generated exception
 */
public synchronized int unmarshal(java.nio.ByteBuffer byteBuffer) throws Exception
{
    super.unmarshal(byteBuffer);

    try
    {
        // attribute objectID marked as not serialized
        objectID.unmarshal(byteBuffer);
        // attribute referencedObjectID marked as not serialized
        referencedObjectID.unmarshal(byteBuffer);
        // attribute updateNumber marked as not serialized
        updateNumber = (short)(byteBuffer.getShort() & 0xFFFF);
        // attribute forceID marked as not serialized
        forceID = ForceID.unmarshalEnum(byteBuffer);
        // attribute modifications marked as not serialized
        modifications.unmarshal(byteBuffer);
        // attribute objectType marked as not serialized
        objectType.unmarshal(byteBuffer);
        // attribute specificObjectAppearance marked as not serialized
        specificObjectAppearance = byteBuffer.getInt();
        // attribute generalObjectAppearance marked as not serialized
        generalObjectAppearance = (short)(byteBuffer.getShort() & 0xFFFF);
        // attribute numberOfPoints marked as not serialized
        numberOfPoints = (short)(byteBuffer.getShort() & 0xFFFF);
        // attribute requesterID marked as not serialized
        requesterID.unmarshal(byteBuffer);
        // attribute receivingID marked as not serialized
        receivingID.unmarshal(byteBuffer);
        // attribute objectLocation marked as not serialized
        for (int idx = 0; idx < numberOfPoints; idx++)
        {
        Vector3Double anX = new Vector3Double();
        anX.unmarshal(byteBuffer);
        objectLocation.add(anX);
        }

    }
    catch (java.nio.BufferUnderflowException bue)
    {
        System.err.println("*** buffer underflow error while unmarshalling " + this.getClass().getName());
    }
    return getMarshalledSize();
}

 /*
  * Override of default equals method.  Calls equalsImpl() for content comparison.
  */
@Override
 public synchronized boolean equals(Object obj)
 {
    if(this == obj)
      return true;

    if(obj == null)
       return false;

    if(!getClass().isAssignableFrom(obj.getClass())) //if(getClass() != obj.getClass())
        return false;

    return equalsImpl(obj);
 }

@Override
 public synchronized boolean equalsImpl(Object obj)
 {
     boolean ivarsEqual = true;

     final ArealObjectStatePdu rhs = (ArealObjectStatePdu)obj;

     if( ! (objectID.equals( rhs.objectID) )) ivarsEqual = false;
     if( ! (referencedObjectID.equals( rhs.referencedObjectID) )) ivarsEqual = false;
     if( ! (updateNumber == rhs.updateNumber)) ivarsEqual = false;
     if( ! (forceID == rhs.forceID)) ivarsEqual = false;
     if( ! (modifications.equals( rhs.modifications) )) ivarsEqual = false;
     if( ! (objectType.equals( rhs.objectType) )) ivarsEqual = false;
     if( ! (specificObjectAppearance == rhs.specificObjectAppearance)) ivarsEqual = false;
     if( ! (generalObjectAppearance == rhs.generalObjectAppearance)) ivarsEqual = false;
     if( ! (requesterID.equals( rhs.requesterID) )) ivarsEqual = false;
     if( ! (receivingID.equals( rhs.receivingID) )) ivarsEqual = false;

     for (int idx = 0; idx < objectLocation.size(); idx++)
        if( ! ( objectLocation.get(idx).equals(rhs.objectLocation.get(idx)))) ivarsEqual = false;

    return ivarsEqual && super.equalsImpl(rhs);
 }

 @Override
 public synchronized String toString()
 {
    StringBuilder sb  = new StringBuilder();
    StringBuilder sb2 = new StringBuilder();
    sb.append(getClass().getSimpleName());
    sb.append(" objectID:").append(objectID); // writeOneToString
    sb.append(" referencedObjectID:").append(referencedObjectID); // writeOneToString
    sb.append(" updateNumber:").append(updateNumber); // writeOneToString
    sb.append(" forceID:").append(forceID); // writeOneToString
    sb.append(" modifications:").append(modifications); // writeOneToString
    sb.append(" objectType:").append(objectType); // writeOneToString
    sb.append(" specificObjectAppearance:").append(specificObjectAppearance); // writeOneToString
    sb.append(" generalObjectAppearance:").append(generalObjectAppearance); // writeOneToString
    sb.append(" requesterID:").append(requesterID); // writeOneToString
    sb.append(" receivingID:").append(receivingID); // writeOneToString
    sb.append(" objectLocation: ");
    objectLocation.forEach(r->{ sb2.append(" ").append(r);}); // writeList
    sb.append(sb2.toString().trim());
    // https://stackoverflow.com/questions/2242471/clearing-a-string-buffer-builder-after-loop
    sb2.setLength(0); // reset

   return sb.toString();
 }
} // end of class
