/**
 * Copyright (c) 2008-2023, MOVES Institute, Naval Postgraduate School (NPS). All rights reserved.
 * This work is provided under a BSD-style open-source license, see project
 * <a href="https://savage.nps.edu/opendis7-java/license.html" target="_blank">license.html</a> and <a href="https://savage.nps.edu/opendis7-java/license.txt" target="_blank">license.txt</a>
 */
// header autogenerated using string template dis7javalicense.txt



package edu.nps.moves.dis7.pdus;

import java.util.*;
import java.io.*;
import edu.nps.moves.dis7.enumerations.*;
import edu.nps.moves.dis7.utilities.PduFactory;

/**
 * Base class of PduBase and LiveEntityPdu
 * @see <a href="https://ieeexplore.ieee.org/document/6387564" target="_blank">IEEE Std 1278.1-2012, IEEE Standard for Distributed Interactive Simulation - Application Protocols</a> 
 */
public abstract class Pdu extends Object implements Serializable,Marshaller
{
   /** The version of the protocol. 5=DIS-1995, 6=DIS-1998, 7=DIS-2012 uid 3 */
   protected DISProtocolVersion protocolVersion = DISProtocolVersion.IEEE_12781_2012;

   /** Exercise ID provides a unique identifier */
   protected byte exerciseID = (byte)0;

   /** Type of pdu, unique for each PDU class uid 4 */
   protected DisPduType pduType = DisPduType.values()[0];

   /** value that refers to the protocol family, eg SimulationManagement, et uid 5 */
   protected DISProtocolFamily protocolFamily = DISProtocolFamily.values()[0];

   /** Timestamp value, int representing number of 1.675 microseconds as interval past hour */
   protected int timestamp;

   /** Length, in bytes, of the PDU */
   protected short length;


/** Constructor creates and configures a new instance object */
 public Pdu()
 {
 }
    /** Create deep copy of current object using PduFactory.
     * @return deep copy of PDU
     */
     public synchronized Pdu copyByPduFactory()
     {
         PduFactory pduFactory = new PduFactory();
         Pdu newPdu = null; // initialize empty as placeholder
         try
         {
             newPdu = pduFactory.createPdu(marshal());
         }
         catch (Exception e)
         {
             System.err.println("Pdu copyByPduFactory() Exception: " + e.toString());
             System.exit(-1); // TODO: Abruptly ending VM not a best practice
         }
         return newPdu;
     }

  /**
   * Returns size of this serialized (marshalled) object in bytes
   * @see <a href="https://en.wikipedia.org/wiki/Marshalling_(computer_science)" target="_blank">https://en.wikipedia.org/wiki/Marshalling_(computer_science)</a>
   * @return serialized size in bytes
   */
public int getMarshalledSize()
{
   int marshalSize = 0; 

   if (protocolVersion != null)
       marshalSize += protocolVersion.getMarshalledSize();
   marshalSize += 1;  // exerciseID
   if (pduType != null)
       marshalSize += pduType.getMarshalledSize();
   if (protocolFamily != null)
       marshalSize += protocolFamily.getMarshalledSize();
   marshalSize += 4;  // timestamp
   marshalSize += 2;  // length

   return marshalSize;
}


/** Setter for {@link Pdu#protocolVersion}
  * @param pProtocolVersion new value of interest
  * @return same object to permit progressive setters */
public synchronized Pdu setProtocolVersion(DISProtocolVersion pProtocolVersion)
{
    protocolVersion = pProtocolVersion;
    return this;
}
/** Getter for {@link Pdu#protocolVersion}
  * @return value of interest */
public DISProtocolVersion getProtocolVersion()
{
    return protocolVersion; 
}

/** Setter for {@link Pdu#exerciseID}
  * @param pExerciseID new value of interest
  * @return same object to permit progressive setters */
public synchronized Pdu setExerciseID(byte pExerciseID)
{
    exerciseID = pExerciseID;
    return this;
}
/** Utility setter for {@link Pdu#exerciseID}
  * @param pExerciseID new value of interest
  * @return same object to permit progressive setters */
public synchronized Pdu setExerciseID(int pExerciseID){
    exerciseID = (byte) pExerciseID;
    return this;
}
/** Getter for {@link Pdu#exerciseID}
  * @return value of interest */
public byte getExerciseID()
{
    return exerciseID; 
}

/** Setter for {@link Pdu#pduType}
  * @param pPduType new value of interest
  * @return same object to permit progressive setters */
public synchronized Pdu setPduType(DisPduType pPduType)
{
    pduType = pPduType;
    return this;
}
/** Getter for {@link Pdu#pduType}
  * @return value of interest */
public DisPduType getPduType()
{
    return pduType; 
}

/** Setter for {@link Pdu#protocolFamily}
  * @param pProtocolFamily new value of interest
  * @return same object to permit progressive setters */
public synchronized Pdu setProtocolFamily(DISProtocolFamily pProtocolFamily)
{
    protocolFamily = pProtocolFamily;
    return this;
}
/** Getter for {@link Pdu#protocolFamily}
  * @return value of interest */
public DISProtocolFamily getProtocolFamily()
{
    return protocolFamily; 
}

/** Setter for {@link Pdu#timestamp}
  * Warning: this method sets a DIS bit pattern
  * @see setTimestampSeconds
  * @see edu.nps.moves.dis7.utilities.DisTime
  * @param pTimestamp new value of interest
  * @return same object to permit progressive setters */
public synchronized Pdu setTimestamp(int pTimestamp)
{
    timestamp = pTimestamp;
    return this;
}
/** Getter for {@link Pdu#timestamp}
  * Warning: this method gets a DIS bit pattern
  * @see getTimestampSeconds
  * @see edu.nps.moves.dis7.utilities.DisTime
  * @return value of interest */
public int getTimestamp()
{
    return timestamp; 
}

/** Setter for {@link Pdu#length}
  * @param pLength new value of interest
  * @return same object to permit progressive setters */
public synchronized Pdu setLength(short pLength)
{
    length = pLength;
    return this;
}
/** Utility setter for {@link Pdu#length}
  * @param pLength new value of interest
  * @return same object to permit progressive setters */
public synchronized Pdu setLength(int pLength){
    length = (short) pLength;
    return this;
}
/** Getter for {@link Pdu#length}
  * @return value of interest */
public short getLength()
{
    return length; 
}

/**
 * Serializes an object to a DataOutputStream.
 * @throws java.lang.Exception if something goes wrong
 * @see java.io.DataOutputStream
 * @param dos the OutputStream
 */
public void marshal(DataOutputStream dos) throws Exception
{
    try 
    {
       protocolVersion.marshal(dos);
       dos.writeByte(exerciseID);
       pduType.marshal(dos);
       protocolFamily.marshal(dos);
       dos.writeInt(timestamp);
       dos.writeShort(length);
    }
    catch(Exception e)
    {
      System.err.println(e);
    }
}

/**
 * Deserializes an object from a DataInputStream.
 * @throws java.lang.Exception if something goes wrong
 * @see java.io.DataInputStream
 * @see <a href="https://en.wikipedia.org/wiki/Marshalling_(computer_science)" target="_blank">https://en.wikipedia.org/wiki/Marshalling_(computer_science)</a>
 * @param dis the InputStream
 * @return marshalled serialized size in bytes
 */
public synchronized int unmarshal(DataInputStream dis) throws Exception
{
    int uPosition = 0;
    try 
    {
        protocolVersion = DISProtocolVersion.unmarshalEnum(dis);
        uPosition += protocolVersion.getMarshalledSize();
        exerciseID = (byte)dis.readUnsignedByte();
        uPosition += 1;
        pduType = DisPduType.unmarshalEnum(dis);
        uPosition += pduType.getMarshalledSize();
        protocolFamily = DISProtocolFamily.unmarshalEnum(dis);
        uPosition += protocolFamily.getMarshalledSize();
        timestamp = dis.readInt();
        uPosition += 4;
        length = (short)dis.readUnsignedShort();
        uPosition += 2;
    }
    catch(Exception e)
    { 
      System.err.println(e); 
    }
    return getMarshalledSize();
}

/**
 * Packs an object into the ByteBuffer.
 * @throws java.nio.BufferOverflowException if byteBuffer is too small
 * @throws java.nio.ReadOnlyBufferException if byteBuffer is read only
 * @see java.nio.ByteBuffer
 * @param byteBuffer The ByteBuffer at the position to begin writing
 * @throws Exception ByteBuffer-generated exception
 */
public void marshal(java.nio.ByteBuffer byteBuffer) throws Exception
{
   protocolVersion.marshal(byteBuffer);
   byteBuffer.put( (byte)exerciseID);
   pduType.marshal(byteBuffer);
   protocolFamily.marshal(byteBuffer);
   byteBuffer.putInt( (int)timestamp);
   byteBuffer.putShort( (short)length);
}

/**
 * Unpacks a Pdu from the underlying data.
 * @throws java.nio.BufferUnderflowException if byteBuffer is too small
 * @see java.nio.ByteBuffer
 * @see <a href="https://en.wikipedia.org/wiki/Marshalling_(computer_science)" target="_blank">https://en.wikipedia.org/wiki/Marshalling_(computer_science)</a>
 * @param byteBuffer The ByteBuffer at the position to begin reading
 * @return marshalled serialized size in bytes
 * @throws Exception ByteBuffer-generated exception
 */
public synchronized int unmarshal(java.nio.ByteBuffer byteBuffer) throws Exception
{
    try
    {
        // attribute protocolVersion marked as not serialized
        protocolVersion = DISProtocolVersion.unmarshalEnum(byteBuffer);
        // attribute exerciseID marked as not serialized
        exerciseID = (byte)(byteBuffer.get() & 0xFF);
        // attribute pduType marked as not serialized
        pduType = DisPduType.unmarshalEnum(byteBuffer);
        // attribute protocolFamily marked as not serialized
        protocolFamily = DISProtocolFamily.unmarshalEnum(byteBuffer);
        // attribute timestamp marked as not serialized
        timestamp = byteBuffer.getInt();
        // attribute length marked as not serialized
        length = (short)(byteBuffer.getShort() & 0xFFFF);
    }
    catch (java.nio.BufferUnderflowException bue)
    {
        System.err.println("*** buffer underflow error while unmarshalling " + this.getClass().getName());
    }
    return getMarshalledSize();
}


/**
 * A convenience method for marshalling to a byte array.
 * This is not as efficient as reusing a ByteBuffer, but it <em>is</em> easy.
 * @return a byte array with the marshalled {@link Pdu}
 * @throws Exception ByteBuffer-generated exception
 */
public synchronized byte[] marshal() throws Exception
{
    byte[] data = new byte[getMarshalledSize()];
    java.nio.ByteBuffer byteBuffer = java.nio.ByteBuffer.wrap(data);
    marshal(byteBuffer);
    return data;
}
 /*
  * Override of default equals method.  Calls equalsImpl() for content comparison.
  */
@Override
 public synchronized boolean equals(Object obj)
 {
    if(this == obj)
      return true;

    if(obj == null)
       return false;

    if(!getClass().isAssignableFrom(obj.getClass())) //if(getClass() != obj.getClass())
        return false;

    return equalsImpl(obj);
 }

 /**
  * Compare all fields that contribute to the state, ignoring
  * transient and static fields, for <code>this</code> and the supplied object
  * @param obj the object to compare to
  * @return true if the objects are equal, false otherwise.
  */
 public synchronized boolean equalsImpl(Object obj)
 {
     boolean ivarsEqual = true;

     final Pdu rhs = (Pdu)obj;

     if( ! (protocolVersion == rhs.protocolVersion)) ivarsEqual = false;
     if( ! (exerciseID == rhs.exerciseID)) ivarsEqual = false;
     if( ! (pduType == rhs.pduType)) ivarsEqual = false;
     if( ! (protocolFamily == rhs.protocolFamily)) ivarsEqual = false;
     if( ! (timestamp == rhs.timestamp)) ivarsEqual = false;
     if( ! (length == rhs.length)) ivarsEqual = false;
    return ivarsEqual;
 }

 @Override
 public synchronized String toString()
 {
    StringBuilder sb  = new StringBuilder();
    StringBuilder sb2 = new StringBuilder();
    sb.append(getClass().getSimpleName());
    sb.append(" protocolVersion:").append(protocolVersion); // writeOneToString
    sb.append(" exerciseID:").append(exerciseID); // writeOneToString
    sb.append(" pduType:").append(pduType); // writeOneToString
    sb.append(" protocolFamily:").append(protocolFamily); // writeOneToString
    sb.append(" timestamp:").append(timestamp); // writeOneToString
    sb.append(" length:").append(length); // writeOneToString

   return sb.toString();
 }

// autogenerated by JavaGenerator.writePduUtilityMethods()

/** Utility setter for {@link Pdu#timestamp} converting double (or float) to
  * Timestamp in seconds at 2^31 - 1 units past top of hour
  * @see setTimestamp
  * @see edu.nps.moves.dis7.utilities.DisTime
  * @param newTimestamp new timestamp in seconds
  * @return same object to permit progressive setters */
public synchronized Pdu setTimestampSeconds(double newTimestamp)
{
    timestamp = (int) ((newTimestamp * 3600.0) / Integer.MAX_VALUE);
    return this;
}      
/** Utility getter for {@link Pdu#timestamp} converting 
  * integer timestamp at 2^31 - 1 units past top of hour to double (or float)
  * @see getTimestamp
  * @see edu.nps.moves.dis7.utilities.DisTime
  * @return fractional timestamp past hour */
public double getTimestampSeconds()
{
    return timestamp * Integer.MAX_VALUE / 3600.0;
}
 /**
  * Whether or not timestamp for this Pdu occurs after timestamp as another Pdu.
  * @param pdu2 second Pdu for comparison
  * @return whether timestamp for this Pdu occurs later
  */
  public boolean occursAfter(Pdu pdu2)
  {
     return (getTimestamp() < pdu2.getTimestamp());
  }
 /**
  * Whether or not timestamp for this Pdu occurs before timestamp as another Pdu.
  * @param pdu2 second Pdu for comparison
  * @return whether timestamp for this Pdu occurs earlier
  */
  public boolean occursBefore(Pdu pdu2)
  {
     return (getTimestamp() < pdu2.getTimestamp());
  }
 /**
  * Whether or not this Pdu occurs at same timestamp as another Pdu.
  * @param pdu2 second Pdu for comparison
  * @return whether timestamps are identical for both Pdus
  */
  public boolean occursSameTime(Pdu pdu2)
  {
     return (getTimestamp() == pdu2.getTimestamp());
  }


} // end of class
